function [F,U] = GetHyperbolicResidual(x)
%Joshua Stough
%COMP 290-89, Pollefeys
%May 6, 2003

%Return the reprojection error graph that the lsqnonlin
%would have seen.

%see HyperbolicMirrorCost.m for a discussion of what's happening.

global x_world x_image


%Build the rotation matrix and translation vector.

%Build rotation as 3-d product of rotx roty and rotz.

c = cos(x(1));
s = sin(x(1));
rotx = [1 0 0; 0 c -s; 0 s c];

c = cos(x(2));
s = sin(x(2));
roty = [c 0 s; 0 1 0; -s 0 c];

c = cos(x(3));
s = sin(x(3));
rotz = [c -s 0; s c 0; 0 0 1];

R = rotx*roty*rotz;

T = x(4:6);

cam_coords = R*x_world;
ncols = size(cam_coords,2);
cam_coords = cam_coords + T*ones(1,ncols);
%cam_coords should be 3xn (50).

%compute the Q (1xn) vector.
Q = sqrt(sum(cam_coords(1:2,:).^2));

%make a and c and z vectors.
A = x(10)*ones(1,ncols);
C = x(11)*ones(1,ncols);
Z = cam_coords(3,:);

%Make square vectors to maybe save some time.
AS = A.^2;
CS = C.^2;
QS = Q.^2;
ZS = Z.^2;
CZ = C.*Z;
QUOT = (4*CS + QS - 4*CZ + ZS);

%T = (-A.^2 + C.^2) * (-2*C.^2 + C.*Z + A.*sqrt(4*C.^2 + Q.^2 - 4*C.*Z + Z.^2));
%T = T ./ (-C.^2 * (-2*C + Z).^2 + A.^2 * (4*C.^2 + Q.^2 - 4*C.*Z + Z.^2));

T = (-AS + CS) .* (-2*CS + CZ + A.*sqrt(QUOT));
T = T ./ (-CS .* (-2*C + Z).^2 + AS .* QUOT);

if (sum(T > 1 | T < 0) > 0)
    disp('found negative or too large t values.');
end

%Now compute the p's, on (0,2c) + t*(Q,z - 2c).
TS = ones(2,1)*T;
P = [zeros(1,ncols); 2*C] + TS.*[Q; (Z - 2*C)];


THETA = atan(P(1,:)./P(2,:));
R = x(12)*tan(THETA/2);



D = cam_coords(1:2,:);
D = D./(ones(2,1)*sqrt(sum(D.^2)));
%the cos of the angle phi is just D(1,:) (dot prod with (1,0)).

COS = D(1,:);
SIN = sqrt(1 - COS.^2);
SIN(D(2,:) < 0) = -SIN(D(2,:) < 0);
%These are the sines and cosines of the phi's we want.  Times
%R give the orthogonal u'

UPRIME = [R.*COS; R.*SIN; ones(1,ncols)];

%Build the K matrix.
Kinv = [1 0 -x(7); 0 x(9) -x(9)*x(8); 0 0 1];

K = inv(Kinv);

U = K*UPRIME;
U = U(1:2,:);

%U are the coords reprojected by the camera model.  Now we
%return |x_image - U|

F = sqrt(sum((x_image - U).^2))';