function [cp] = PickCalibrationPoints(image, num_lines, num_circles, scale)
%Joshua Stough
%COMP 290-89, Multiple View Geometry, Pollefeys
%April 10, 2003

%This program lets the user select calibration points
%according to the cylinder experiment in "Calibration
%of a fish eye lens with field of view larger than 180."

%The program will ask users to select points on the
%circles.  Since the circles are calibrated, and this is
%a picture of these calibrated cirlces, we should then 
%be able to take these points and compute the camera
%matrix for Marc's fish eye camera.

if(nargin < 4), scale = 5; end
%Scale because if the image doesn't fit in matlab, we need
%to see smaller, and resize the numbers.
if(nargin < 3), num_circles = 5; end
if(nargin < 2), num_lines = 10; end


disp('This program lets you specify the points');
disp('used to determine the camera matrix of ');
disp('a camera with a fish eye lens.  Click on');
disp('on the points in order, starting at the ');
disp('obvious starting point and spiralling in.');

%greyscale the image, so that the selected points can
%be easily spotted.

if (size(image,3) > 1)
    I = double(rgb2gray(image));
else
    I = double(image);
end

I = I/max(max(I));

s = size(I);
Overlay = zeros(s(1),s(2),3);

Overlay(:,:,1) = I;
Overlay(:,:,2) = I;
Overlay(:,:,3) = I;

%The picture is multichanneled so that the overlay of
%selected point pairs is visible.

%Display the image.
h = figure('Name','Image'); 
imagesc(I); colormap('gray');

cp = zeros(2, num_lines, num_circles);

disp('Specify the num_lines cross points on each circle');
disp('and your results will be plotted.');

quitter = 0;

for i = 1:num_circles    
    
    for j = 1:num_lines
        [x,y] = ginput(1);
        
        if (isempty(x))
            disp('You have finished with only selecting an incomplete list.');
            quitter = 1;
            break;
        end
        
        temp = round(y);
        y = round(x);
        x = temp;
        
        cp(:,j,i) = [x y]';
        
        disp(sprintf('(%f , %f)', x, y));
        
        if (x > 1 & y > 1 & x < s(1) & y < s(2))
            Overlay(x-1,y-1,1) = 1;
            Overlay(x-1,y,1) = 1;
            Overlay(x-1,y+1,1) = 1;
            Overlay(x,y-1,1) = 1;
            %Overlay(x,y,1) = 1;
            Overlay(x,y+1,1) = 1;
            Overlay(x+1,y-1,1) = 1;
            Overlay(x+1,y,1) = 1;
            Overlay(x+1,y+1,1) = 1;
            Overlay(x,y+2,1) = 1;
            Overlay(x,y-2,1) = 1;
            Overlay(x-2,y,1) = 1;
            Overlay(x+2,y,1) = 1;
            
            Overlay(x-1,y-1,3) = 1;
            Overlay(x-1,y,3) = 1;
            Overlay(x-1,y+1,3) = 1;
            Overlay(x,y-1,3) = 1;
            %Overlay(x,y,3) = 1;
            Overlay(x,y+1,3) = 1;
            Overlay(x+1,y-1,3) = 1;
            Overlay(x+1,y,3) = 1;
            Overlay(x+1,y+1,3) = 1;
            Overlay(x,y+2,3) = 1;
            Overlay(x,y-2,3) = 1;
            Overlay(x-2,y,3) = 1;
            Overlay(x+2,y,3) = 1;
        end
    end
    
    if (quitter == 1)
        break;
    end
end


imagesc(Overlay);